
% FHCGMTT: fast hierarchical clustering based on growing multi-layer
% topology training. 4 neurons are trained for each sub-map with
% membership control (each node contains sqrt(n) data points or less)

function [Node,Child,CorSet,Layer]=CGMTTMain(X,Pm)

%% preprocessing
X=X(1:Pm.XlthP,:);

% train a node for layer 1
Node{1}=X(randperm(Pm.XlthP,1),:);
for i=1:Pm.XlthP
    Node{1}=Node{1}+Pm.LearnRate0*(X(i,:)-Node{1});
end

% pre-assign some values
Layer(1,1)=1; % map the layer No. and the sequential No. of each nodes
Layer(2,1)=2; 
Child{1}=1; % record if have child node for each nodes. 1: has child, 0: no child.
CorSet{1}=true(1,Pm.Xlth); % at the beginning, corset of the first node is the whole data set
CorSet{1}(Pm.XlthP+1:Pm.Xlth)=false;
StopGrow=0; % determiner to decide if to grow the topology or not
PreNode=1; % sequential number of the just processed node
ThisCoarse=1; % sequential number of the coarse node in the just processed layer
%CoarsePool=1; % the set of coarse node in the just processed layer
ThisLayer=2; % sequential number of the layer to be created
ThisNode=2; % sequential number of the node to be created
Counter=2; % No. of existing node

%% growing training
% recursively grow and train the topology
while ThisLayer~=1
    if StopGrow==0 % 0: grow deeper layer, 1: stop grow
        SubDataLct=find(CorSet{PreNode}(ThisCoarse,:)~=0); % extract the "SubData" for deeper layer growth
        SubData=X(SubDataLct,:);
        [Node{ThisNode},CorSet{ThisNode},NumNode] = ...
            CTrainNodes(SubData,SubDataLct,Pm); % ***function*** %
        Child{ThisNode}=zeros(1,NumNode);
    end
    Bad1=find(sum(CorSet{ThisNode},2)>Pm.UpLimitP); % find the node with high membership
    Bad2=find(Child{ThisNode}==0); % find the node with no heir
    CoarsePool=intersect(Bad1,Bad2);
    if isempty(CoarsePool)==1 % back to check the upper layer
        ThisLayer=ThisLayer-1;
        ThisNode=PreNode;
        PreNode=find(Layer==ThisLayer-1, 1, 'last' );
        StopGrow=1;
    else % if nodes have no heir and too many data points
        ThisCoarse=CoarsePool(1);
        Counter=Counter+1;
        Child{ThisNode}(ThisCoarse)=1;
        PreNode=ThisNode;
        ThisNode=Counter;
        ThisLayer=ThisLayer+1;
        Layer(ThisNode,1)=ThisLayer;
        StopGrow=0;
    end
end
end




